/******************************* AT_DMA96.C *********************************

Copyright Symbios Logic January 1992

Author:		Ken Stewart Symbios Logic Products Applications Engineer
Purpose:	53CF90 test software

This software was written by Symbios Logic to develop and test new products.  
It is not a general SCSI driver.  Symbios Logic assumes no liability for
its use, misuse or inability to use.

Project files:
		CF90.C 	Main body of code
		TARGET96.C	Target Mode routines
		CF90ISR.C	Interrupt Service Routine
		CF90TEST.C	Automatic Test Module
		AT_DMA96.C	hardware dependant DMA stuff
		AT_DMA96.H
		CF90AT.H	hardware addresses, global defines
		CF90EV.H	external variables

								  **********/

#include <stdio.h>
#include <dos.h>
#include "at_dma90.h"
#include "cf90at.h"
#include "cf90ev.h"


/****************** Initialize IBM microchannel DMA channel 3 ****************

initIBMdma(*start_addrs, numbytes, rw)

Input arguments:
start_addrs	= a pointer to the first element in the memory array which
		  will be accessed by the DMA controller
numbytes	= number of bytes to transfer
rw		= direction of DMA transfer: r (for read) or w (write).  Read
		  is an SCSI read, or Data In, which requires the DMA to write
		  to memory.  So, rw is backward for this module.

calls:		calc_abs_adrs(array_name) 	to calculate the absolute
						address of the memory array

!!! Machine Dependant Code !!!

This routine sets up the DMA controler on the AT (clone) motherboard to
transferdata between the 53CF90 and main memory via the motherboard DMA.

							   ***************/


void initIBMdma(void far *start_addrs, unsigned long numbytes, char rw)

{
calc_abs_adrs(start_addrs);	/* get array address for DMA controler */

/* next two lines for troubleshooting; delete later
printf("\n\n  - DMA start address = %06lX",abs_adrs);
printf("\n  - msB mdB lsB = %02X %02X %02X",msB,mdB,lsB);  */

outportb(dma_request,ch3);	/* clear software request bit          */
outportb(dma_mask,(0x40|ch3));	/* mask channel 3 while programming it */

outportb(dma_flop,0);		/* reset byte pointer for memory address */
outportb(dma3_addrs,lsB);	/* LSB of memory destination address     */
outportb(dma3_addrs,mdB);	/* middle byte of 20-bit address         */
outportb(dma3_page,msB);	/* MSB of memory destination address     */

outportb(dma_flop,0);				/* reset byte pointer    */
outportb(dma3_count,(numbytes) & 0xff);			/* LSB of transfer count */
outportb(dma3_count,((numbytes) >> 8) & 0xff);	/* MSB of transfer count */

if (parms.burst=='y')
  switch (rw)
	{
	case 'r' :  outportb(dma_mode,(demand|WT|ch3));  /* write demand mode */
	break;
	case 'w' :  outportb(dma_mode,(demand|RD|ch3));  /* read demand mode */
	break;
	default  :  outportb(dma_mode,(demand|RD|ch3));  /* read demand mode */
	}

else
  switch (rw)
	{
	case 'r' :  outportb(dma_mode,(single|WT|ch3));  /* write single */
	break;
	case 'w' :  outportb(dma_mode,(single|RD|ch3));  /* read single transfer */
	break;
	default  :  outportb(dma_mode,(single|RD|ch3));  /* read single transfer */
	}

  outportb(dma_mask,ch3);			  /* un-mask channel 3 */
}

/********************** Calculate Absolute Address **************************

calc_abs_adrs(*ptr)

input argument:   ptr	= 32-bit pointer to the first element of the memory
			array which will be accessed by the DMA.  In C the
			array name IS the pointer to the first element, so
			the subscript is not necessary.

!!! Machine Dependant Code !!!

This routine translates Intel microprocessor segment:offset into a 20-bit
absolute address.  The external functions FP_SEG() and FP_OFF() are in the
Borland Turbo C Library; intended for use on PC, AT, and MicroChannel
architectures.  This routine will not be necessary on machines
(such as Motorala) that have linear memory addressing.

							   ***************/

void calc_abs_adrs (void far *ptr)
{

extern unsigned long	seg, off, abs_adrs;

seg = FP_SEG(ptr);			/* Far Pointer Segment */
off = FP_OFF(ptr);			/* Far Pointer Offset */

abs_adrs = ((seg << 4) + off);		/* 20-bit address */

/* Normally, msB >> 16, mdB >> 8 and lsB >> zero.  But to get the AT to
   transfer words using the byte oriented 8237 DMA controller, IBM
   connected system address bit 1 to DMA address bit 0.  So mdB and lsB
   must be shifted right by one extra bit.  Also, they added an 8-bit
   latch (called the page register) which creats a 24-bit address from the
   16-bit address generated by the 8237.  Page latch shifting is not
   affected by this byte-to-word hardware trick.  */

#if 0
msB = abs_adrs >> 16;                   /* Most Significant byte */
mdB = abs_adrs >>  9;                   /* Middle Byte */
lsB = abs_adrs >>  1;                   /* Least Significant Byte */
#else
msB = abs_adrs >> 16;                   /* Most Significant byte */
mdB = abs_adrs >>  8;                   /* Middle Byte */
lsB = abs_adrs >>  0;                   /* Least Significant Byte */
#endif
}


