/******************************** CF90TEST.C *********************************

Copyright Symbios Logic 12 October 1988

Author:		Ken Stewart Symbios Logic Products Applications Engineer
Purpose:	53CF90 test software

This software was written by Symbios Logic to develop and test new products.  
It is not a general SCSI driver.  Symbios Logic assumes no liability for
its use, misuse or inability to use.

Project files:
		CF90_3.C	Main body of code
		TARGET90.C	Target Mode routines
		CF90ISR.C	Interrupt Service Routine
		CF90TEST.C	Automatic Test Module
		AT_DMA90.C	hardware dependant DMA stuff
		AT_DMA90.H
		CF90AT.H	hardware addresses, global defines
		CF90EV.H  external variables

				  **************/
#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include "cf90at.h"
#include "cf90ev.h"
#include <time.h>


/************************* 53CF90 Automatic Test ******************************

This is a test program that calls existing routines in the C94.C routine.

rev 0.0	20 Feb 1991	start
rev 0.1 08 Apr 1991	port to 53C90 test vehicle
rev 0.1 18 Jun 1991	add odd byte test
rev 0.1  9 Sept  91     add do while pass_count !=0, toggle ATN
rev 0.2 14 Feb 1992	add T8 loop

								  **********/

/****************************** Test Driver **********************************/
void test_driver (void)
{
unsigned long	pass_count;
unsigned int	test_word;
long		elapsed_days, elapsed_hours, elapsed_minutes, elapsed_time,
		start_time, current_time;
struct	date	today;				/* most recent date */
struct	time	now;				/* most recent time */
struct	date	start;				/* start of test date */
struct	time	begin;				/* start of test time */

pass_count = 0;
test_word = 0;
test_error = false;
parms.detail = 'n';				/* turn off verbose mode */

getdate(&start);
gettime(&begin);
time(&start_time);

do {

getdate(&today);
gettime(&now);

time(&current_time);
elapsed_time = current_time - start_time;

elapsed_days    = elapsed_time / 86400l;
elapsed_time    = elapsed_time - (elapsed_days * 86400l);

elapsed_hours   = elapsed_time / 3600;
elapsed_time    = elapsed_time - (elapsed_hours * 3600);

elapsed_minutes = elapsed_time / 60;


clrscr();
printf("\n\n*+*+*+*+*+*+*+*+*+*+*+*+*+*  53C90 Automatic Test *+*+*+*+*+*+*+*+*+*+*+*+*+*");
printf("\n\npass count      = %12lu d",pass_count);

printf("\tStart test on %02d/%02d/%04d",start.da_mon,start.da_day,start.da_year);
printf("  at %02d:%02d",begin.ti_hour,begin.ti_min);

printf("\t\t\t\t\t\tcurrent loop  %02d/%02d/%04d",today.da_mon,today.da_day,today.da_year);
printf("  at %02d:%02d",now.ti_hour,now.ti_min);

printf("\n\t\t\t\t\t%d days",elapsed_days);
printf("  %d hours",elapsed_hours);
printf("  %d minutes",elapsed_minutes);


printf("\nSelect with ATN = ");
if (parms.atn == 'n') printf("No");
else printf("Yes");
parms.msg_byte = 0xc0;				/* disconnect privlidge */

load_regs();
FIFO_Test();
if (test_error)
  {
  do
    {
    printf("\n\nspace bar to continue or q to quit");
    temp =getch();
    } while (temp != ' ' && temp != 'q' && temp != 'Q');
  if (temp == 'q' || temp == 'Q')
    {
    pause_flag = false;
    return;
    }
  }

x_count_test();
if (test_error)
  {
  do
    {
    printf("\n\nspace bar to continue or q to quit");
    temp =getch();
    } while (temp != ' ' && temp != 'q' && temp != 'Q');
  if (temp == 'q' || temp == 'Q')
    {
    pause_flag = false;
    return;
    }
  }

asynch_test();
if (test_error)
  {
  do
    {
    printf("\n\nspace bar to continue or q to quit");
    temp =getch();
    } while (temp != ' ' && temp != 'q' && temp != 'Q');
  if (temp == 'q' || temp == 'Q')
    {
    pause_flag = false;
    return;
    }
  }

dma_cdb_test();
if (test_error)
  {
  do
    {
    printf("\n\nspace bar to continue or q to quit");
    temp =getch();
    } while (temp != ' ' && temp != 'q' && temp != 'Q');
  if (temp == 'q' || temp == 'Q')
    {
    pause_flag = false;
    return;
    }
  }

//odd_test();
if (test_error)
  {
  do
    {
    printf("\n\nspace bar to continue or q to quit");
    temp =getch();
    } while (temp != ' ' && temp != 'q' && temp != 'Q');
  if (temp == 'q' || temp == 'Q')
    {
    pause_flag = false;
    return;
    }
  }
synch_test(0);				/* aligned boundarys */
if (test_error)
  {
  do
    {
    printf("\n\nspace bar to continue or q to quit");
    temp =getch();
    } while (temp != ' ' && temp != 'q' && temp != 'Q');
  if (temp == 'q' || temp == 'Q')
    {
    pause_flag = false;
    return;
    }
  }

if (RCB == 0x11 || RCB == 0xa2)	/* ESRB not in all versions */
  {
  parms.blkcount = 3;
  parms.blksize = 512;
  byte_count = parms.blksize * parms.blkcount;

  synch_test(1);			/* misaligned scatter gather */
  if (test_error)
    {
    do
      {
      printf("\n\nspace bar to continue or q to quit");
      temp =getch();
      } while (temp != ' ' && temp != 'q' && temp != 'Q');
    if (temp == 'q' || temp == 'Q')
      {
      pause_flag = false;
      return;
      }
    }
  }

++pass_count;

/**** Test word :

	     Alt-DMA   T-8     ATN
		0	0	0
                0	0	1
                0	1	0
                0	1	1
		1	0	0	illegal
                1	0	1	illegal
                1	1	0
                1	1	1

****/

if (test_word == 7) test_word = 0;		/* max count, reset */
else ++test_word;
if ((test_word & 4) == 4)
  test_word = test_word | 2;	/* eliminate illegal case */

if ((test_word & 1) == 1) parms.atn = 'y';
else parms.atn = 'n';

if ((test_word & 2) == 2) parms.T8  = 'y';
else parms.T8 = 'n';

if ((test_word & 4) == 4) parms.burst = 'y';
else parms.burst = 'n';


} while (pass_count !=0 && test_error == false);

printf("\n\n\n All tests complete");
pause_flag = true;
end:;
}	/* end of test_driver */


/***************************** Asynchronous Test *************************/

void asynch_test(void)
{
test_error = false;
printf("\nInitiator Asynchronous test .............. ");

sync_nego(200,0);				/* negotiate aynchronous */
if (scsi_error)
  {
  printf("\n  asynchronous negotiation failed");
  test_error=true;
  return;
  }

fillbuff('i',0x10);			/* Init buffer starting at 10 hex */
EXSCSI('w');					/* do a SCSI write */
if (scsi_error)
  {
  printf("\n  ERROR in sending data!");
  test_error = true;
  return;
  }

fillbuff('d',0xff);				/* change buffer */

EXSCSI('r');					/* SCSI read */
if (scsi_error)
  {
  printf("\n  ERROR in reading data!");
  test_error = true;
  return;
  }

check_buf(0x10);
if (test_error) printf("FAIL\n  Data In not equal to Data Out");

else printf("pass");
}


/****************************** Synchronous Test *************************
synch_test(type)

inputs:	type 	= 0	data aligned on word boundaries
		= 1	incoming data scattered to misaligned boundaries

				  ***********/

void synch_test(int type)
{
int		max_offset, max_period, min_period, increment;
unsigned int	period, offset, start;


test_error = false;

switch(type)
  {
    case 0:
    printf("\nInitiator Synchronous test ...                            ");
    break;

    case 1:
    printf("\nMisaligned Scatter/Gather ....                            ");
    break;

    default:
      {
      printf("\ninput argument error to synch_test(%d)",type);
      test_error = true;
      }
  }

start = 0x20;			/* Init buffer starting at 20 hex */

if (parms.burst == 'y') max_offset = 7;
else max_offset = 15;

if (parms.FASTclk == 'y')			/* 40 MHz clock */
  {
  max_period = 875;				/* period in nanoseconds */
  min_period = 100;
  increment = 25;
  }
else						/* 25 MHz clock */
  {
  max_period = 1000;				/* period in nanoseconds */
  min_period = 200;
  increment = 40;
  }

for (period=max_period; period>=min_period; period=period-increment)
  {
  if (parms.burst == 'y' || parms.T8 == 'y') max_offset = 7;
  else max_offset = 15;
  for(offset=max_offset; offset>0; --offset)
   {
   sync_nego(period, offset);			/* negotiate synchronous */
   if (scsi_error)
     {
     if (period <= 200)
       {
       printf("\n  Target cannot support FAST SCSI transfer rate");
       scsi_error = false;			/* continue testing */
       return;
       }
     else
       {
       printf("\n\n  failed synchronous negotiation");
       test_error = true;
       return;
       }
     }


   printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
   printf(" Period = %04d  Offset = %02d ",period,offset);

   fillbuff('i',start);
   EXSCSI('w');					/* SCSI write */
    if (scsi_error)
      {
      printf("\n  ERROR in sending data!");
      test_error = true;
      return;
      }

    fillbuff('d',0xff);				/* change buffer */

    if (type == 0) EXSCSI('r');	/* aligned SCSI read */
   // else esrb();			/* misaligned scatter-gather */

    if (scsi_error)
      {
      printf("\n  ERROR in reading data!");
      test_error = true;
      return;
      }

    if (type == 0) check_buf(start);			/* compare data */
    else check_buf(start-1);

    if (test_error || scsi_error)
      {
      printf("FAIL\n  Data In not equal to Data Out");
      return;
      }

    ++start;
    }  /* end of for(offset) loop */
  }    /* end of for(period) loop */
    printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
    printf("                          ");
    printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
    printf("............ pass");
}	/* end of synch_test() */



/****************************** DMA the CDB Test **************************/

void dma_cdb_test(void)
{
test_error = false;
printf("\nDMA the CDB Test ......................... ");

sync_nego(200,0);				/* negotiate aynchronous */
if (scsi_error)
  {
  printf("\n  asynchronous negotiation failed");
  test_error=true;
  return;
  }

fillbuff('i',0x50);				/* Init buffer starting at 50 hex */

DMA_CDB('w');				/* DMA the CDB for SCSI write */
if (scsi_error)
  {
  printf("\n  ERROR! in sending data");
  test_error=true;
  return;
  }

fillbuff('d',0xff);				/* change buffer */

DMA_CDB('r');				/* DMA the CDB for SCSI read */
if (scsi_error)
  {
  printf("\n  ERROR! in reading data");
  test_error=true;
  return;
  }

check_buf(0x50);
if (test_error) printf("FAIL\n  Data In not equal to Data Out");
else printf("pass");
}

/***************************** Check Buffer *******************************
check_buf(start)

inputs:		start = starting value of expected incrementing pattern
output:		none
modifies:	test_error

				  **********/

void check_buf(unsigned int start)
{
unsigned int	k, element, expect;


test_error = false;

for (k=0; k<byte_count; ++k)
  {
  element = dmabyte[k] & 0x0ff;
  expect  = (k + start) & 0x0ff;
  if (element != expect)
    {
    if (test_error == false)		/* only need one error message */
      {
      printf("ERROR!");
      printf("\n  error beginning at location %04X",k);
      printf("\n  buffer start = %04X",start);
      printf("\n  buffer value = %04X    expected %04X\n",element,expect);
      }
    test_error = true;
    }
  }
}
/***************************** Exectue SCSI command *********************/
void EXSCSI(char rw)
{
if (rw == 'w') cdb[0]=0x0a;	/* SCSI Read Command 		*/
if (rw == 'r') cdb[0]=8;   	/* else write command           */
cdb[1]=0x00;			/* lun = 0, msbaddr = 0		*/
cdb[2]=0x01;			/* msb address = 01		*/
cdb[3]=0x00;			/* lsb address = 00		*/
cdb[4]=parms.blkcount;		/* one block to transfer	*/
cdb[5]=0x00;			/* reserved byte		*/

DATA_DRIVER (rw, 6, byte_count); 	/* driver for data transfer */
}

